/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): Christophe Demarey.

====================================================================*/

package DiningPhilosophers.cif;

import DiningPhilosophers.*;

/**
 * This is the implementation of the OMG IDL3 DiningPhilosophers::Observer 
 * component type.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * This class inherits from the local CCM_Observer interface
 * generated by the OpenCCM's IDL3 to IDL2 mapping generator.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a> 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 2.0
 */

public class ObserverImpl
     extends DiningPhilosophers.ObserverServiceComposition.ObserverService_impl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Stores philosopher's states. */
    private java.util.Hashtable philosopherStates_;

    /** The main GUI. */
    private javax.swing.JFrame frame_;

    /** The philosopher's panels. */
    private java.util.Hashtable philosopherPanels_;

    /** The box panel used to store philosopher's panel GUI. */
    private javax.swing.Box box_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ObserverImpl()
    {
        philosopherStates_ = new java.util.Hashtable();
        philosopherPanels_ = new java.util.Hashtable();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL Components::EnterpriseComponent
    //
    // ==================================================================

    /**
     * Complete the component configuration and launch the
     * observer GUI.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        // Instantiating the GUI.

        // Creates a Swing Frame.
        frame_ = new javax.swing.JFrame("The Observer's GUI");
        // Sets its size.
        frame_.setSize(400, 300);

        // Constructs and shows the GUI.
        box_ = javax.swing.Box.createVerticalBox();
        frame_.getContentPane().add(box_);
        frame_.pack();
        frame_.show();
    }

    // ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        // Release the associated frame.
        frame_.dispose();
        frame_ = null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL dinner::StatusInfoConsumer
    //
    // ==================================================================

    /**
     * To receive DiningPhilosophers::StatusInfo events.
     *
     * Update the observer GUI with received status informations.
     */
    public synchronized void
    push(StatusInfo event)
    {
        if (! philosopherPanels_.containsKey(event.name)) {
            PhilosopherPanel panel = new PhilosopherPanel(event);
            box_.add(panel);
            philosopherPanels_.put(event.name, panel);
            frame_.pack();
            frame_.show();
        } else {
            PhilosopherPanel panel;
            panel = (PhilosopherPanel)philosopherPanels_.get(event.name);
            panel.updatePanel(event);
        }
    }
}
